/*----------------------------------------------------------------------*/
/* initapp.c - setting up main window in X                              */
/*   pg 162 X window system programming                                 */
/*----------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define DEF_APP_GLOB	/* .. so globals get defined */
#include "app_glob.c"

#define DEFAULT_BGCOLOR		"white"
#define DEFAULT_FGCOLOR		"black"
#define DEFAULT_BDWIDTH		1
#define DEFAULT_FONT		"fixed"

typedef struct APP_PARAMS
{
	char *name;
	char **p_value_string;
} APP_PARAMS;

/* .. Default parameter values (as strings) */
char	*theBGcolor	=DEFAULT_BGCOLOR,
	*theFGcolor	=DEFAULT_FGCOLOR,
	*theFont	=DEFAULT_FONT,
	*theGeom_rsrc	=NULL,
	*display_name	=NULL,
	*theGeom_cline	=NULL,	
	*theGeom	=NULL;

/* .. List of user-customizable resources */

APP_PARAMS app_resources[]=
{
	"background"	, &theBGcolor,
	"foreground"	, &theFGcolor,
	"font"		, &theFont,
	"geometry"	, &theGeom_rsrc,
};

int num_resources = sizeof(app_resources) /sizeof(APP_PARAMS);

/* .. List of command line options */

APP_PARAMS app_options[] =
{
	"-display",  &display_name,
	"-d",        &display_name,
	"-geometry", &theGeom_cline,
	"-g",        &theGeom_cline,
};

int num_options = sizeof(app_options)/sizeof(APP_PARAMS);

void usage(void);

/* -------------------------------------------------------- */
void initapp(int argc, char **argv, int x , int y, 
  unsigned width, unsigned height)
{
  int				i, j, screen_num, screen_depth;
  char				*tmpstr;
  Colormap			default_cmap;
  XColor			color;
  int				bitmask;
  XGCValues			gcv;
  XSetWindowAttributes		xswa;
  XWMHints			xwmh;
  XSizeHints			xsh;
  char				default_geometry[80];

  theAppName = argv[0];
  AppDone=0;

  /* .. Parse command-line options. Because each option has a value,
   * they are processed in pairs.  */
  for (i=1;i<argc;i+=2) {
    for(j=0;j<num_options;j++) {
      if(strcmp(app_options[j].name,argv[i]) ==0) {
	*app_options[j].p_value_string = argv[i+1];
	break;
      }
    }
  }

  /* .. Open connection to display selected by user */

  if ((theDisplay = XOpenDisplay(display_name)) == NULL ) {
    fprintf(stderr,"%s : can't open display named %s \n",
      argv[0],XDisplayName(display_name));
    exit(1);
  }

  /* .. Get resources from the source file */

  for (i=0; i< num_resources;i++) {
    if((tmpstr=XGetDefault(theDisplay,theAppName,
      app_resources[i].name)) != NULL)
    *app_resources[i].p_value_string = tmpstr;
  }

  /* .. Set up colors and fonts */

  if ((theFontStruct = XLoadQueryFont(theDisplay, theFont)) == NULL) {
    fprintf(stderr,"%s: display %s cannot load font %s\n",
      theAppName, DisplayString(theDisplay), theFont);
    exit(1);
  }

  /* .. Select colors using the default colormap */

  default_cmap = DefaultColormap(theDisplay, DefaultScreen(theDisplay));

  /* .. Main windows background color */
  if (XParseColor(theDisplay, default_cmap, theBGcolor, &color) == 0 ||
    XAllocColor(theDisplay,default_cmap,&color) == 0)

    /* .. Use white background in case of failure */
    theBGpix=WhitePixel(theDisplay, DefaultScreen(theDisplay));
  else theBGpix = color.pixel;

  /* .. Main window's foreground color */
  if (XParseColor(theDisplay, default_cmap, theFGcolor, &color) == 0 ||
    XAllocColor(theDisplay,default_cmap,&color) == 0)
    /* .. Use black in case of failure */
    theFGpix=BlackPixel(theDisplay, DefaultScreen(theDisplay));
  else theFGpix = color.pixel ;

  /* .. Fill out a XsizeHints structure to inform the window manager
   * of desired size and location of main window
  */
  xsh.flags = (PPosition | PSize | PMinSize);
  xsh.height = height ;
  xsh.min_height = xsh.height ;
  xsh.width = width ;
  xsh.min_width = xsh.width ;
  xsh.x = x ;
  xsh.y = y;

  /* .. Construct a Default geometry string */
  sprintf(default_geometry,"%dx%d+%d+%d", xsh.width, xsh.height,xsh.x,xsh.y);
  theGeom = default_geometry;

  /* .. Overide the geometry if necessary */
  if(theGeom_cline != NULL) theGeom = theGeom_cline ;
  else if(theGeom_rsrc != NULL) theGeom = theGeom_rsrc ;

  /* .. Process the geometry specifications */
  bitmask = XGeometry(theDisplay, DefaultScreen(theDisplay),
    theGeom, default_geometry, DEFAULT_BDWIDTH,
    theFontStruct -> max_bounds.width,
    theFontStruct -> max_bounds.ascent + theFontStruct -> max_bounds.descent,
    1, 1, &(xsh.x), &(xsh.y), &(xsh.width), &(xsh.height));

  /* .. Check bitmask and set flags in XSizeHints structure */
  if (bitmask & (XValue | YValue)) xsh.flags |= USPosition;
  if (bitmask & (WidthValue | HeightValue)) xsh.flags |= USSize;

  /* .. Create the main window using position and size
   * information derived above.  For border color, use the
   * foreground color
   */
  theMain = XCreateSimpleWindow(theDisplay, DefaultRootWindow(theDisplay),
    xsh.x, xsh.y, xsh.width, xsh.height, DEFAULT_BDWIDTH, theFGpix, theBGpix);

  screen_num = DefaultScreen(theDisplay);
  screen_depth = DefaultDepth(theDisplay,screen_num);
  lPixmap = XCreatePixmap(theDisplay,theMain,xsh.width,xsh.height,screen_depth);
  /* .. Set Window Manager properties (window_name, icon
   * name, command-line, and size hints.
   */
  XSetStandardProperties(theDisplay, theMain, theAppName,
    theAppName, None, argv, argc, &xsh);

  /* .. Give other hints to window manager by filing out an
   * XWMHints structure and calling XSetWMHints()
   */
  xwmh.flags = (InputHint|StateHint);
  xwmh.input = True;
  xwmh.initial_state = NormalState ;

  XSetWMHints (theDisplay, theMain, &xwmh);

  /* .. Finally, create a graphics context for the main window */
  gcv.font = theFontStruct -> fid ;
  gcv.foreground = theFGpix ;
  gcv.background = theBGpix ;
  theGC = XCreateGC(theDisplay, theMain,
    (GCFont | GCForeground | GCBackground), &gcv);

  gcv.function = GXcopy;
  pixGC = XCreateGC(theDisplay, lPixmap, GCFunction, &gcv);
  /* .. Set main windows attributes (color_map, bit_gravity) */
  xswa.colormap = DefaultColormap(theDisplay, DefaultScreen(theDisplay));
  xswa.bit_gravity = NorthWestGravity ;
  XChangeWindowAttributes(theDisplay, theMain, (CWColormap | CWBitGravity), 
    &xswa);

  /* .. Select exposure events for the main window */
  XSelectInput(theDisplay, theMain, ExposureMask);

  /* .. Map the main window */
  XMapWindow(theDisplay, theMain);

  /* .. Get the windows attributes and save them for future use */
  if(XGetWindowAttributes(theDisplay, theMain, &MainXWA) == 0) {
    fprintf(stderr, "Error getting attributes of Main. \n");
    exit(2);
  }
}

/* ---------------------------------------------------------------------*/
 
